<?php


namespace FiloBlu\Hotfix\Plugin\Magento\Catalog\Controller\Adminhtml\Product;

class Validate
{

    const LOG_NAME = "/var/log/FiloBlu_Hotfix_Image_Cleaner.log";
    const ACTIVE = "hotfix/image_cleaner/active";
    const IMAGE_TAGS = "hotfix/image_cleaner/image_tags";

    private $_resource;
    private $connection;

    private $row_id;
    private $sku;
    private $name;

    private $scopeConfig;

    public function __construct(
        \Magento\Framework\App\ResourceConnection $resource,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ){
        $this->_resource = $resource;
        $this->scopeConfig = $scopeConfig;
    }

    public function beforeExecute(
        \Magento\Catalog\Controller\Adminhtml\Product\Validate $subject
    ) {
        if(!$this->isActive()){
            return $subject;
        }

        $productData = $subject->getRequest()->getPost('product', []);
        if($this->thereIsAnyImageToRemove($productData)){
            $this->executeImageCleaner();
        }
        return $subject;
    }

    private function thereIsAnyImageToRemove($productData){

        $this->sku = $productData['sku'];
        $this->name = $productData['name'];
        if(isset($productData['media_gallery']['images']) && count($productData['media_gallery']['images'])>0){
            foreach ($productData['media_gallery']['images'] as $post_image_id => $post_image) {
                if (isset($post_image['removed']) && $post_image['removed'] == 1){
                    $this->row_id = $post_image['row_id'];
                    return true;
                    break;
                }
            }
        }
        return false;
    }

    private function executeImageCleaner() {

        if($this->row_id){
            $this->writeLog("Detected image removed from the backend! Cleaning images from product gallery '{$this->name}' (SKU: {$this->sku}) (ROW_ID: {$this->row_id}) if storeview != 0");
        } else {
            $this->writeLog("Detected image removed from the backend! Cleaning images from all products galleries if storeview != 0");
        }

        $image_tags = $this->getImageTagsString();

        if(!$image_tags){
            $this->writeLog("Can't clean from product gallery '{$this->name}' (SKU: {$this->sku}) (ROW_ID: {$this->row_id}) - missing image tags from config!");
            return false;
        }

        $sql = "DELETE FROM catalog_product_entity_varchar WHERE store_id <> 0 AND attribute_id IN (
                    SELECT attribute_id 
                    FROM eav_attribute 
                    WHERE backend_type = 'varchar' 
                    AND frontend_input = 'media_image' 
                    AND entity_type_id = 4 
                    AND attribute_code IN ({$image_tags})
                ) ";

        if($this->row_id){
            $sql .= " AND row_id = ({$this->row_id})";
        }

        //$this->writeLog($sql);
        $this->getConnection()->query($sql);

        $sql = "DELETE FROM catalog_product_entity_media_gallery_value WHERE store_id <> 0 ";

        if($this->row_id){
            $sql .= " AND row_id = ({$this->row_id})";
        }

        //$this->writeLog($sql);
        $this->getConnection()->query($sql);

        return true;

    }

    private function getConnection()
    {
        if (!$this->connection) {
            $this->connection = $this->_resource->getConnection('core_write');
        }
        return $this->connection;
    }

    private function writeLog($msg){
        $writer = new \Zend\Log\Writer\Stream(BP . self::LOG_NAME);
        $logger = new \Zend\Log\Logger();
        $logger->addWriter($writer);
        $logger->info($msg);
    }

    private function isActive(){
        return $this->scopeConfig->getValue(self::ACTIVE);
    }

    private function getImageTagsString(){
        $string = $this->scopeConfig->getValue(self::IMAGE_TAGS);
        $array_string_exploded = explode(",",$string);

        if(count($array_string_exploded) == 0){
            return "";
        }

        foreach ($array_string_exploded as &$item) {
            $item = trim($item);
            $item = "'".$item."'";
        }
        $final_string = implode(",",$array_string_exploded);
        return $final_string;
    }
}
